

## This function calculates the rate-weighted area under the ROC curve (rAUC)
## This version calculates the rAUC in rate-accuracy space
##
## written by Louise AC Millard louise.millard@bristol.ac.uk
##
## Please cite:
## Millard, Louise AC, Peter A. Flach, and Julian PT Higgins. 
## "Rate-constrained ranking and the rate-weighted AUC." 
## Machine Learning and Knowledge Discovery in Databases. Springer Berlin Heidelberg, 2014. 386-403.

# see examples.r for sample function calls
# in labels: 1 is positive (high score is good), 0 is negative (low score is good)

rauc <- function(scores, labels, w) {
    
    ## sort the labels by score
	orderIdx = order(-s);
	labels = labels[,orderIdx];
	labels = rbind(labels);
	
    nTotal = ncol(labels);
    nTotalPlus = sum(labels);
    nTotalMinus = nTotal - nTotalPlus;
	
    pi0 = nTotalPlus/(nTotalPlus + nTotalMinus);
    pi1 = nTotalMinus/(nTotalPlus + nTotalMinus);
    
    aucAbsolute = 0;
    
	nPlus = 0;
	nMinus = 0;
    
    tiesPlus = 0;
    tiesMinus = 0;
    
    # init tie values for first example
	prevTiesScore = -1;

    for (i in 1:nTotal) {
        
        if (prevTiesScore == scores[i]) { # tieing
            if (labels[i] >0) {
                tiesPlus = tiesPlus + 1;
            }
            else {
                tiesMinus = tiesMinus + 1;
            }
        }
        else {
            
            #### either no longer tieing or never tied so update the masses
            #### with the previous index values and set current ties values

            #### update auc mass for this last `step' in the ranking
            saucRes = sauc(aucAbsolute, tiesMinus, tiesPlus, nMinus,nPlus, nTotal, nTotalMinus, nTotalPlus, pi0, pi1, w);
            
            nMinus = saucRes$nMinus;
            nPlus = saucRes$nPlus;
            aucAbsolute = saucRes$aucAbsolute;
            
            if (labels[i] > 0) {
                tiesPlus = 1;
                tiesMinus = 0;
            }
            else {
                tiesPlus = 0;
                tiesMinus = 1;
            }
    
            # set the new ties scores
            prevTiesScore = scores[i];
            
        }
        
    }
    
    #### update auc mass for the last `step' in the ranking
    saucRes = sauc(aucAbsolute, tiesMinus, tiesPlus, nMinus,nPlus, nTotal, nTotalMinus, nTotalPlus, pi0, pi1, w);
    nMinus = saucRes$nMinus;
    nPlus = saucRes$nPlus;
    aucAbsolute = saucRes$aucAbsolute;
    
    
    ####
    #### calculate weighted mass of whole area in rate-accuracy space
	#### bounds depends on class distribution 
    if (pi0<=pi1) {
    	f1 <- function(r) w(r)*2*r;
    	f2 <- function(r) w(r)*2*pi0;
    	f3 <- function(r) w(r)*2*(1-r);
        areaTotal = integrate(f1,0,pi0)$value+integrate(f2,pi0,pi1)$value + integrate(f3,pi1,1)$value;
	}    
	else {
		f1 <- function(r) w(r)*2*r
		f2 <- function(r) w(r)*2*pi1
		f3 <- function(r) w(r)*2*(1-r)
        areaTotal = integrate(f1,0,pi1)$value+integrate(f2,pi1,pi0)$value + integrate(f3,pi0,1)$value;
    }
    
    rauc = aucAbsolute/areaTotal;
    return(rauc);
    
}


