

## The rAUC function calculates the rate-weighted area under the ROC curve (rAUC)
## This version calculates the rAUC in rate-accuracy space
##
## written by Louise AC Millard louise.millard@bristol.ac.uk
##
## Please cite:
## Millard, Louise AC, Peter A. Flach, and Julian PT Higgins. 
## "Rate-constrained ranking and the rate-weighted AUC." 
## Machine Learning and Knowledge Discovery in Databases. Springer Berlin Heidelberg, 2014. 386-403.


# aucAbsolute - The current value of the AUC mass, which this function
# updates for the current `step' in the ranking

# The step is between rank (nMinus+nPlus) and 
# (nMinus+nPlus+tiesMinus+tiesPlus)
# If there is more than one example in this step (because there are tied
# examples with the same score), then the area is added in the chunks:
# 1. All ties minus vertical slices (in one go)
# 2. All ties plus vertical slices (in one go)
# 3. The ties triangle
# It is necessary to do it in this order so that the correct function 
# corresponds to the correct `move' - the way the ties triangle is shaped
# means that underneath it the negative moves have to be imagined to 
# precede the positive moves, even though they are really tied. This is 
# just a formality to easily construct the mass of the ties triangle.

# tiesMinus - The Number of negatives in this ties segment
# tiesPlus - The Number of positives in this ties segment
# nMinus - The Number of negatives procressed thus far in total
# nPlus - The Number of positives procressed thus far in total
# nTotalMinus - The total number of examples in the dataset
# nTotalMinus - The total number of negative examples in the dataset
# nTotalPlus - The total number of positive examples in the dataset
# pi0 - Class distribution - proportion of positives
# pi1 - Class distribution - proportion of negatives
# w - a PDF function used to weight across the rates


sauc <- function(aucAbsolute, tiesMinus, tiesPlus, nMinus,nPlus, nTotal, nTotalMinus, nTotalPlus, pi0, pi1, w) {

	# step across rate-accuracy space for negative examples
    if (tiesMinus>=1) {
        nMinusPrev = nMinus;
        nMinus = nMinus + tiesMinus;
        aROC = vauc(nMinusPrev,nPlus, nMinus,nPlus, nTotal, nTotalMinus, nTotalPlus, pi0, pi1, 0, w);
        aucAbsolute = aucAbsolute + aROC;
    }
    
	# step across rate-accuracy space for positive examples
    if (tiesPlus>=1) {
        nPlusPrev = nPlus;
        nPlus = nPlus + tiesPlus;
        aROC = vauc(nMinus,nPlusPrev, nMinus,nPlus, nTotal, nTotalMinus, nTotalPlus, pi0, pi1, 1, w);
        aucAbsolute = aucAbsolute + aROC;
    }
            
    ## last ties triangle
    # ties triangle across previous slice
    if (tiesPlus>=1 && tiesMinus >=1) {
        t = tauc(nMinus, nPlus, tiesMinus, tiesPlus, nTotalMinus, nTotalPlus, pi0, pi1, w);
        aucAbsolute = aucAbsolute + t;
    }
    
    
    return(list("nMinus"=nMinus, "nPlus"=nPlus, "aucAbsolute"=aucAbsolute));

}


