

## The rAUC function calculates the rate-weighted area under the ROC curve (rAUC)
## This version calculates the rAUC in rate-accuracy space
##
## written by Louise AC Millard louise.millard@bristol.ac.uk
##
## Please cite:
## Millard, Louise AC, Peter A. Flach, and Julian PT Higgins. 
## "Rate-constrained ranking and the rate-weighted AUC." 
## Machine Learning and Knowledge Discovery in Databases. Springer Berlin Heidelberg, 2014. 386-403.

# nMinus - The Number of negatives procressed thus far in total when this
# ties segment ends
# nPlus - The Number of positives procressed thus far in total when this
# ties segment ends
# tiesMinus - The Number of negatives in this ties segment
# tiesPlus - The Number of positives in this ties segment
# nTotalMinus - The total number of negative examples in the dataset
# nTotalPlus - The total number of positive examples in the dataset
# pi0 - Class distribution - proportion of positives
# pi1 - Class distribution - proportion of negatives
# w - a PDF function used to weight across the rates

# See diagram in paper for the points and angles

tauc <- function(nMinus, nPlus, tiesMinus, tiesPlus, nTotalMinus, nTotalPlus, pi0, pi1, w) {

	###
	### the 3 end points of the ties triangle 
	
	point1.f0 = (nPlus - tiesPlus)/nTotalPlus;
	point1.f1 = (nMinus - tiesMinus)/nTotalMinus;
	point1.a = pi0*point1.f0 + pi1*(1 - point1.f1);
	point1.r = pi0*point1.f0 + pi1*point1.f1;
	
	point2.f0 = (nPlus - tiesPlus)/nTotalPlus;
	point2.f1 = nMinus/nTotalMinus;
	point2.a = pi0*point2.f0 + pi1*(1 - point2.f1);
	point2.r = pi0*point2.f0 + pi1*point2.f1;
	
	point3.f0 = nPlus/nTotalPlus;
	point3.f1 = nMinus/nTotalMinus;
	point3.a = pi0*point3.f0 + pi1*(1 - point3.f1);
	point3.r = pi0*point3.f0 + pi1*point3.f1;
	
	
	#### finding lengths and angles of the two sub triangles
	
	line1 = sqrt((point1.a - point2.a)^2 + (point2.r - point1.r)^2);
	line2 = sqrt((point3.a - point2.a)^2 + (point3.r - point2.r)^2);
	
	aRadians = atan(line1/line2);
	aDegrees = aRadians*180/pi;
	
	# in rate-accuracy space the grid is always squares not parallelograms
	b = 45;
	
	# the remaining angle in triangle 2
	cDegrees = 180 - (b + aDegrees);
	cRadians = cDegrees*pi/180;
	
	# Use sine rule to find accuracy 'height' between triangles 1 and 2
	h = sin(aRadians) * line2 / sin(cRadians);
	
	####
	#### t1 and t2 are two sub triangles of a single ties triangle
	
	# for triangle 1 the triangle height increases as the rate increases
	
	t1F <- function(r) w(r)*h*(r-point1.r)/(point2.r-point1.r);
	t1 = integrate(t1F, point1.r, point2.r);
	
	# for triangle 2 the triangle height decreases as the rate increases
	t2F <- function(r) w(r)*h*(1-(r-point2.r)/(point3.r-point2.r));
	t2 = integrate(t2F, point2.r, point3.r);
	
	tauc = t1$value + t2$value;
	return(tauc);
}
